//
//  zoomdemoViewController.m
//  zoomdemo
//
//  Created by Michael Heyeck on 3/27/10.
//  Copyright Fair Oaks Labs, Inc. 2010. All rights reserved.
//

#import "zoomdemoViewController.h"


const static CGSize tileSize = {320, 460};
const static NSInteger labelTag = 101;


@interface zoomdemoViewController ()

@property (nonatomic, retain) UIScrollView* scrollView;

- (void)removeTiles;
- (void)createTiles;
- (UIView*)addTileForFrame:(CGRect)frame;
- (void)sizeContent;
- (void)reload;

@end


@implementation zoomdemoViewController

@synthesize content;

- (UIScrollView*)scrollView
{
	return (UIScrollView*) self.view;
}

- (void)setScrollView:(UIScrollView*)newScrollView
{
	self.view = newScrollView;
}


/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad
{
    [super viewDidLoad];

	world = self.scrollView.frame.size;
	scale = .9;
	
	[self createTiles];
	tileBox = CGRectZero;
	
	[self sizeContent];
	self.scrollView.contentOffset = CGPointMake(-.05*self.scrollView.frame.size.width,
												-.05*self.scrollView.frame.size.height);
	[self.scrollView flashScrollIndicators];
}


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc
{
	// Clear weak delegate reference
	if (self.scrollView.delegate == self)
	{
		self.scrollView.delegate = nil;
	}
	
	[self removeTiles];
	[content release];
    [super dealloc];
}

#pragma mark Scroll view delegate methods

- (void)scrollViewDidScroll:(UIScrollView*)scrollView
{
	// Tile width and height (in view space)
	CGFloat w = tileSize.width;
	CGFloat h = tileSize.height;
	
	// Legal region (in view space)
	CGRect legal = CGRectMake(0, 0, world.width*scale, world.height*scale);
	
	// Displayed region (in view space)
	CGRect viewport = CGRectIntersection([self.content convertRect:self.scrollView.bounds fromView:self.scrollView], legal);
	
	// For existing tiles
	for (UIView* tv in [self.content subviews])
	{
		// If there's no overlap with viewport, toss the tile
		if (!CGRectIntersectsRect(viewport, tv.frame))
		{
			[tv removeFromSuperview];
			[extraTiles addObject:tv];
		}
	}
	
	NSInteger oldMinCol = tileBox.origin.x;
	NSInteger oldMinRow = tileBox.origin.y;
	NSInteger oldMaxCol = CGRectGetMaxX(tileBox)-1;
	NSInteger oldMaxRow = CGRectGetMaxY(tileBox)-1;
	
	// For new tiles
	tileBox = CGRectIntegral(CGRectApplyAffineTransform(viewport, CGAffineTransformMakeScale(1/w, 1/h)));
	for (int bw = tileBox.size.width, n = bw*tileBox.size.height-1; n >= 0; n--)
	{
		NSInteger c = tileBox.origin.x + n % bw;
		NSInteger r = tileBox.origin.y + n / bw;
		
		//	If missing, create
		if ((c < oldMinCol) || (c > oldMaxCol) || (r < oldMinRow) || (r > oldMaxRow))
		{
			UIView* tv = [self addTileForFrame:CGRectIntersection(CGRectMake(c*w, r*h, w, h), legal)];
			[(UILabel*)[tv viewWithTag:labelTag] setText:[NSString stringWithFormat:@"(%d, %d)",c,r]];
		}
	}
}


- (UIView*)viewForZoomingInScrollView:(UIScrollView*)scrollView
{
	return content;
}


- (void)scrollViewDidEndZooming:(UIScrollView*)scrollView withView:(UIView*)view atScale:(float)relScale
{
	// Cache offset before zoom scale is reset
	CGPoint offset = self.scrollView.contentOffset;
	
	// Transfer zoom from scroll view to world->view transform; resize content at the new scale
	self.scrollView.zoomScale = 1.0;
	scale *= relScale;
	[self sizeContent];
	
	// Restore the old offset
	self.scrollView.contentOffset = offset;
	
	// Re-render
	[self reload];
	
	// Flash
	[self.scrollView flashScrollIndicators];
}

#pragma mark Extension methods

- (void)removeTiles
{
	for (UIView* tv in tiles)
	{
		// Remove tiles from content view
		[tv removeFromSuperview];
	}
	[tiles release];
	tiles = nil;
	
	[extraTiles release];
	extraTiles = nil;
}


- (void)createTiles
{
	if (tiles) [self removeTiles];
	
	tiles = [[NSMutableArray alloc] initWithCapacity:4];
	extraTiles = [[NSMutableSet alloc] initWithCapacity:4];
}


- (UIView*)addTileForFrame:(CGRect)frame
{
	UIView* tv = [extraTiles anyObject];
	if (tv)
	{
		// tv also retained by tiles
		[extraTiles removeObject:tv];
	}
	else
	{
		tv = [[UIView new] autorelease];
		tv.clearsContextBeforeDrawing = NO;
		tv.backgroundColor = [UIColor grayColor];
		[tiles addObject:tv];
		
		UILabel* label = [[[UILabel alloc] initWithFrame:CGRectMake(0, 0, tileSize.width, 18)] autorelease];
		label.backgroundColor = [UIColor clearColor];
		label.font = [UIFont systemFontOfSize:18];
		label.tag = labelTag;
		[tv addSubview:label];
	}
	
	[self.content addSubview:tv];
	tv.frame = frame;
	[tv setNeedsDisplay];
	return tv;
}


- (void)sizeContent
{
	self.content.frame = CGRectMake(0, 0, world.width*scale, world.height*scale);
	self.scrollView.contentSize = CGSizeMake(CGRectGetMaxX(self.content.frame),
											 CGRectGetMaxY(self.content.frame));
}


- (void)reload
{
	// Recycle all tiles
	for (UIView* tv in [self.content subviews])
	{
		[tv removeFromSuperview];
		[extraTiles addObject:tv];
	}
	tileBox = CGRectZero;
	
	// Trigger re-tiling
	[self scrollViewDidScroll:self.scrollView];
}

@end
